@@ -29,6 +29,10 @@
 #include <windows.h>
 #include <prsht.h>
 
+#ifndef _WIN_CE
+#include <winioctl.h>
+#endif
+
 #endif
 
 #ifndef _WIN_CE
@@ -71,7 +75,7 @@
     #include <emx/syscalls.h>
   #endif
 #else
-  #ifdef _MSC_VER
+  #if defined(_MSC_VER) || defined(__MINGW32__)
       #include <exception>
   #else
     #include <except.h>
@@ -196,7 +200,7 @@
 	#endif
 #endif
 
-#if defined(__sparc) || defined(sparc)
+#if defined(__sparc) || defined(sparc) || defined(__hpux)
   #ifndef BIG_ENDIAN
      #define BIG_ENDIAN
   #endif
