@@ -9,7 +9,7 @@ typedef unsigned int     uint32; //32 bits exactly
 typedef          int     sint32; //signed 32 bits exactly
 #define PRESENT_INT32
 
-#if defined(_WIN_32) || defined(__GNUC__) || defined(__sgi) || defined(_AIX) || defined(__sun)
+#if defined(_WIN_32) || defined(__GNUC__) || defined(__sgi) || defined(_AIX) || defined(__sun) || defined(__hpux)
 typedef wchar_t wchar;
 #else
 typedef ushort wchar;
